% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_read_sensorlog.R
\name{acti_read_sensorlog}
\alias{acti_read_sensorlog}
\alias{acti_sensorlog_csv_spec}
\alias{acti_sensorlog_csv_colnames_mapping}
\title{Read SensorLog Data}
\usage{
acti_read_sensorlog(file, verbose = FALSE, robust = FALSE)

acti_sensorlog_csv_spec()

acti_sensorlog_csv_colnames_mapping()
}
\arguments{
\item{file}{A character vector of SensorLog files, usually from unzipping
the file}

\item{verbose}{print diagnostic messages.  Either logical or integer, where
higher values are higher levels of verbosity.}

\item{robust}{if \code{TRUE} then \link{acti_rewrite_sensorlog_csv} is run on the data
to try to fix any shifts with the data.}
}
\value{
A \code{data.frame} of data
}
\description{
Read SensorLog Data
}
\examples{
library(actiread)
file = acti_example_sensorlog_file()
df = acti_read_sensorlog(file)
head(df)
}
