% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_read_sensorlogger_zzz.R
\name{acti_convert_sensorlogger_time}
\alias{acti_convert_sensorlogger_time}
\title{Convert vectors ensuring no new NA}
\usage{
acti_convert_sensorlogger_time(x)
}
\arguments{
\item{x}{a vector}
}
\value{
A converted `vector` the same length as `x` or errors if there
are introduced NAs.
}
\description{
Convert vectors ensuring no new NA
}
