% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_read_gt3x.R
\name{acti_read_gt3x}
\alias{acti_read_gt3x}
\alias{acti_info_gt3x}
\title{Read GT3X file}
\usage{
acti_read_gt3x(
  path,
  asDataFrame = TRUE,
  imputeZeroes = TRUE,
  verbose = TRUE,
  ...,
  fill_zeroes = TRUE,
  apply_tz = FALSE,
  check_attributes = TRUE,
  tz = "GMT"
)

acti_info_gt3x(path, ...)
}
\arguments{
\item{path}{Path to gt3x file}

\item{asDataFrame}{convert to an \code{activity_df}, see
\code{as.data.frame.activity}}

\item{imputeZeroes}{Impute zeros in case there are missingness?
Default is \code{FALSE}, in which case
the time series will be incomplete in case there is missingness.}

\item{verbose}{print diagnostic messages, higher values = more verbosity.}

\item{...}{additional arguments to pass to \code{\link[read.gt3x:read.gt3x]{read.gt3x::read.gt3x()}}}

\item{fill_zeroes}{Rows with all zeros will be filled in with the last
observation carried forward as is done with ActiLife.  Recommended}

\item{apply_tz}{Apply the timezone from the header \code{TimeZone} attribute,
if available}

\item{check_attributes}{Check that the attributes are included This is a sanity check,
including checking that \code{sample_rate} is in the attributes.}

\item{tz}{timezone to project the data into.  The data read in via
\code{\link[read.gt3x:read.gt3x]{read.gt3x::read.gt3x()}} says the timezone is GMT, but the time values is in the
native timezone.  So this data is projected into the correct time zone and then
forced into the timezone given by \code{tz}.  Set to \code{NULL} to not apply this
forcing.}
}
\value{
A \code{data.frame}
}
\description{
Read GT3X file
}
\examples{
library(actiread)
data = acti_read_gt3x(acti_example_gt3x())
data = acti_read_gt3x(
  acti_example_gt3x(),
  tz = NULL,
  apply_tz = FALSE,
  verbose = FALSE,
  fill_zeroes = FALSE
)
info = acti_info_gt3x(acti_example_gt3x())
}
