% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_read_sensorlogger_location.R,
%   R/acti_read_sensorlogger_zzz.R
\name{acti_sensorlogger_location_colnames_mapping}
\alias{acti_sensorlogger_location_colnames_mapping}
\alias{acti_sensorlogger_location_spec}
\alias{acti_read_sensorlogger_location}
\alias{acti_read_sensorlogger}
\alias{acti_read_sensorlogger_general}
\title{Read SensorLogger Data}
\usage{
acti_sensorlogger_location_colnames_mapping()

acti_sensorlogger_location_spec()

acti_read_sensorlogger_location(file, ...)

acti_read_sensorlogger(file, verbose = FALSE, ...)

acti_read_sensorlogger_general(file, ..., verbose = FALSE)
}
\arguments{
\item{file}{A character vector of SensorLogger files, usually from unzipping
the file, or a zip file of SensorLogger files}

\item{...}{additional arguments to pass to \code{\link[readr:read_csv]{readr::read_csv()}}.
If \code{verbose = FALSE}, then \code{progress = FALSE} and \code{show_col_types = FALSE},
unless otherwise overridden}

\item{verbose}{print diagnostic messages.  Either logical or integer, where
higher values are higher levels of verbosity.}
}
\value{
A \code{data.frame} of data
}
\description{
Read SensorLogger Data
}
