% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_utils.R
\name{tzoffset_to_tz}
\alias{tzoffset_to_tz}
\title{Transform timezone offset to timezone}
\usage{
tzoffset_to_tz(x)
}
\arguments{
\item{x}{A character vector}
}
\value{
A character vector
}
\description{
Transform timezone offset to timezone
}
\examples{
tzoffset_to_tz(c("+00:00", "-05:00", "+01:00"))
}
