% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_example_data.R
\name{acti_example_gt3x}
\alias{acti_example_gt3x}
\alias{acti_example_cwa}
\alias{acti_example_sensorlog_file}
\alias{acti_example_sensorlogger_file}
\alias{acti_example_sensorlogger_location_file}
\title{Activity/Actigraphy Example Data}
\usage{
acti_example_gt3x()

acti_example_cwa()

acti_example_sensorlog_file()

acti_example_sensorlogger_file()

acti_example_sensorlogger_location_file()
}
\value{
A file path
}
\description{
Activity/Actigraphy Example Data
}
\examples{
library(actiread)
acti_example_gt3x()
acti_example_cwa()
acti_example_sensorlogger_file()
}
