% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/acti_read_cwa.R
\name{acti_read_cwa}
\alias{acti_read_cwa}
\title{Read CWA File}
\usage{
acti_read_cwa(
  path,
  start = 0,
  end = Inf,
  tz = "UTC",
  ...,
  apply_tz = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{path}{Path to cwa file}

\item{start}{where to start in the file, passed to \link[GGIRread:readAxivity]{GGIRread::readAxivity}}

\item{end}{where to end in the file, passed to \link[GGIRread:readAxivity]{GGIRread::readAxivity}}

\item{tz}{time zone for the data \code{time}, passed to \code{desiredtz} argument in
\link[GGIRread:readAxivity]{GGIRread::readAxivity}.  If NULL or \code{""}, no time conversion is done.}

\item{...}{additional arguments to pass to \code{\link[GGIRread:readAxivity]{GGIRread::readAxivity()}}}

\item{apply_tz}{turn the \code{time} column into a \code{POSIXct} and apply the
timezone}

\item{verbose}{print diagnostic messages, higher values = more verbosity.}
}
\value{
A \code{tibble} with attributes of a header, sample rate, and
transformations
}
\description{
Read CWA File
}
\examples{
data = acti_read_cwa(acti_example_cwa())
data = acti_read_cwa(
  acti_example_cwa(),
  tz = NULL,
  apply_tz = FALSE,
  verbose = FALSE
)
}
